@@ -138,22 +138,52 @@ gdbm_file_seek (GDBM_FILE dbf, off_t off, int whence)
 #endif
 }
 
+#ifdef GDBM_FAILURE_ATOMIC
+/* From gdbmsync.c */
+extern int _gdbm_snapshot(GDBM_FILE);
+#endif /* GDBM_FAILURE_ATOMIC */
+
+static inline void
+_gdbmsync_init (GDBM_FILE dbf)
+{
+#ifdef GDBM_FAILURE_ATOMIC
+  dbf->snapfd[0] = dbf->snapfd[1] = -1;
+  dbf->eo = 0;
+#endif
+}
+
+static inline void
+_gdbmsync_done (GDBM_FILE dbf)
+{
+#ifdef GDBM_FAILURE_ATOMIC
+  if (dbf->snapfd[0] >= 0)
+    close (dbf->snapfd[0]);
+  if (dbf->snapfd[1] >= 0)
+    close (dbf->snapfd[1]);
+#endif
+}
+
 static inline int
 gdbm_file_sync (GDBM_FILE dbf)
 {
+  int r = 0;  /* return value */
 #if HAVE_MMAP
-  return _gdbm_mapped_sync (dbf);
+  r = _gdbm_mapped_sync (dbf);
 #elif HAVE_FSYNC
   if (fsync (dbf->desc))
     {
       GDBM_SET_ERRNO (dbf, GDBM_FILE_SYNC_ERROR, TRUE);
-      return 1;
+      r = 1;
     }
-  return 0;
 #else
   sync ();
   sync ();
-  return 0;
 #endif
+#ifdef GDBM_FAILURE_ATOMIC
+  /* If and only if the conventional fsync/msync/sync succeeds,
+     attempt to clone the data file. */
+  if (r == 0)
+    r = _gdbm_snapshot(dbf);
+#endif /* GDBM_FAILURE_ATOMIC */
+  return r;
 }
-
